/*
 * Copyright (C) 2012, Eric Wong <normalperson@yhbt.net>
 * License: GPLv3 or later (see COPYING for details)
 */
#include "cmogstored.h"

/* path must be a free()-able pointer */
struct mog_fd *
mog_file_open_read(struct mog_svc *svc, char *path)
{
	struct mog_fd *mfd;
	struct mog_file *mfile;
	int fd = mog_open_read(svc, path);

	if (fd < 0) return NULL;

	mfd = mog_fd_get(fd);
	mfd->fd = fd;
	mfd->fd_type = MOG_FD_TYPE_FILE;
	mfd->queue_state = MOG_QUEUE_STATE_NEW;

	mfile = &mfd->as.file;
	memset(mfile, 0, sizeof(struct mog_file));
	mfile->fsize = -1;
	mfile->svc = svc;

	/* TODO: we shouldn't force this on callers... */
	mfile->path = path;

	return mfd;
}

/* path must be a free()-able pointer */
struct mog_fd *
mog_file_open_put(struct mog_svc *svc, char *path, int flags)
{
	struct mog_fd *mfd;
	struct mog_file *mfile;
	int fd = mog_open_put(svc, path, flags);

	if (fd < 0) return NULL;

	mfd = mog_fd_get(fd);
	mfd->fd = fd;
	mfd->fd_type = MOG_FD_TYPE_FILE;
	mfd->queue_state = MOG_QUEUE_STATE_NEW;

	mfile = &mfd->as.file;
	memset(mfile, 0, sizeof(struct mog_file));
	mfile->svc = svc;

	return mfd;
}

void mog_file_close(struct mog_fd *mfd)
{
	struct mog_file *mfile = &mfd->as.file;

	assert(mfd->fd_type == MOG_FD_TYPE_FILE && "mog_fd is not a file");

	/* all of these may already be NULL */
	free(mfile->path);
	free(mfile->tmppath);
	free(mfile->md5);

	mog_close(mfd->fd);
}
